==> gitlab-232.9559.62-sources.jar: org/jetbrains/plugins/gitlab/mergerequest/ui/details/model/GitLabMergeRequestStatusViewModel.kt <==
// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package org.jetbrains.plugins.gitlab.mergerequest.ui.details.model

import com.intellij.collaboration.async.launchNow
import com.intellij.collaboration.async.modelFlow
import com.intellij.collaboration.ui.codereview.details.data.CodeReviewCIJob
import com.intellij.collaboration.ui.codereview.details.data.CodeReviewCIJobState
import com.intellij.collaboration.ui.codereview.details.model.CodeReviewStatusViewModel
import com.intellij.collaboration.util.resolveRelative
import com.intellij.openapi.diagnostic.thisLogger
import com.intellij.util.childScope
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.*
import org.jetbrains.plugins.gitlab.api.GitLabServerPath
import org.jetbrains.plugins.gitlab.api.dto.GitLabCiJobDTO
import org.jetbrains.plugins.gitlab.api.dto.GitLabPipelineDTO
import org.jetbrains.plugins.gitlab.mergerequest.data.GitLabCiJobStatus
import org.jetbrains.plugins.gitlab.mergerequest.data.GitLabMergeRequest
import java.net.URI

class GitLabMergeRequestStatusViewModel(
  parentCs: CoroutineScope,
  mergeRequest: GitLabMergeRequest,
  private val serverPath: GitLabServerPath
) : CodeReviewStatusViewModel {
  private val cs = parentCs.childScope()

  private val pipeline: Flow<GitLabPipelineDTO?> = mergeRequest.pipeline

  override val hasConflicts: SharedFlow<Boolean> = mergeRequest.hasConflicts.modelFlow(cs, thisLogger())

  override val ciJobs: SharedFlow<List<CodeReviewCIJob>> = pipeline.map {
    it?.jobs?.map { job -> job.convert() } ?: emptyList()
  }.modelFlow(cs, thisLogger())

  private val _showJobsDetailsRequests = MutableSharedFlow<List<CodeReviewCIJob>>()
  override val showJobsDetailsRequests: SharedFlow<List<CodeReviewCIJob>> = _showJobsDetailsRequests

  override fun showJobsDetails() {
    cs.launchNow {
      val jobs = ciJobs.first()
      _showJobsDetailsRequests.emit(jobs)
    }
  }

  private fun GitLabCiJobDTO.convert(): CodeReviewCIJob {
    val jobUrl: URI = serverPath.toURI().resolveRelative(webPath)
    return CodeReviewCIJob(name, status.toCiState(), jobUrl.toString())
  }

  private fun GitLabCiJobStatus.toCiState(): CodeReviewCIJobState = when (this) {
    GitLabCiJobStatus.CANCELED,
    GitLabCiJobStatus.CREATED,
    GitLabCiJobStatus.MANUAL,
    GitLabCiJobStatus.PREPARING,
    GitLabCiJobStatus.SCHEDULED,
    GitLabCiJobStatus.SKIPPED,
    GitLabCiJobStatus.RUNNING,
    GitLabCiJobStatus.WAITING_FOR_RESOURCE,
    GitLabCiJobStatus.FAILED -> CodeReviewCIJobState.FAILED
    GitLabCiJobStatus.PENDING -> CodeReviewCIJobState.PENDING
    GitLabCiJobStatus.SUCCESS -> CodeReviewCIJobState.SUCCESS
  }
}

==> gitlab-232.10203.2-sources.jar: org/jetbrains/plugins/gitlab/mergerequest/ui/details/model/GitLabMergeRequestStatusViewModel.kt <==
// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package org.jetbrains.plugins.gitlab.mergerequest.ui.details.model

import com.intellij.collaboration.async.launchNow
import com.intellij.collaboration.async.modelFlow
import com.intellij.collaboration.ui.codereview.details.data.CodeReviewCIJob
import com.intellij.collaboration.ui.codereview.details.data.CodeReviewCIJobState
import com.intellij.collaboration.ui.codereview.details.model.CodeReviewStatusViewModel
import com.intellij.collaboration.util.resolveRelative
import com.intellij.openapi.diagnostic.thisLogger
import com.intellij.util.childScope
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.flow.*
import org.jetbrains.plugins.gitlab.api.GitLabServerPath
import org.jetbrains.plugins.gitlab.api.dto.GitLabCiJobDTO
import org.jetbrains.plugins.gitlab.api.dto.GitLabPipelineDTO
import org.jetbrains.plugins.gitlab.mergerequest.data.GitLabCiJobStatus
import org.jetbrains.plugins.gitlab.mergerequest.data.GitLabMergeRequest
import java.net.URI

class GitLabMergeRequestStatusViewModel(
  parentCs: CoroutineScope,
  mergeRequest: GitLabMergeRequest,
  private val serverPath: GitLabServerPath
) : CodeReviewStatusViewModel {
  private val cs = parentCs.childScope()

  private val pipeline: Flow<GitLabPipelineDTO?> = mergeRequest.pipeline

  override val hasConflicts: SharedFlow<Boolean> = mergeRequest.hasConflicts.modelFlow(cs, thisLogger())

  override val ciJobs: SharedFlow<List<CodeReviewCIJob>> = pipeline.map {
    it?.jobs?.map { job -> job.convert() } ?: emptyList()
  }.modelFlow(cs, thisLogger())

  private val _showJobsDetailsRequests = MutableSharedFlow<List<CodeReviewCIJob>>()
  override val showJobsDetailsRequests: SharedFlow<List<CodeReviewCIJob>> = _showJobsDetailsRequests

  override fun showJobsDetails() {
    cs.launchNow {
      val jobs = ciJobs.first()
      _showJobsDetailsRequests.emit(jobs)
    }
  }

  private fun GitLabCiJobDTO.convert(): CodeReviewCIJob {
    val jobUrl: URI = serverPath.toURI().resolveRelative(webPath)
    return CodeReviewCIJob(name, status.toCiState(), jobUrl.toString())
  }

  // TODO: Add more states (CodeReviewCIJobState.SKIPPED -> MANUAL, SKIPPED)
  private fun GitLabCiJobStatus.toCiState(): CodeReviewCIJobState = when (this) {
    GitLabCiJobStatus.CANCELED,
    GitLabCiJobStatus.FAILED,
    GitLabCiJobStatus.MANUAL,
    GitLabCiJobStatus.SKIPPED -> CodeReviewCIJobState.FAILED

    GitLabCiJobStatus.CREATED,
    GitLabCiJobStatus.PENDING,
    GitLabCiJobStatus.PREPARING,
    GitLabCiJobStatus.RUNNING,
    GitLabCiJobStatus.SCHEDULED,
    GitLabCiJobStatus.WAITING_FOR_RESOURCE -> CodeReviewCIJobState.PENDING

    GitLabCiJobStatus.SUCCESS -> CodeReviewCIJobState.SUCCESS
  }
}
